@@ -66,6 +66,7 @@ wm_new(int argc, char **argv)
    w->main_client    = NULL; /* Points to the current 'main' client
 				- eg the 'BIG' window - */
 
+   w->n_active_ping_clients    = 0;
    w->next_click_is_not_double = True;
 
    sattr.event_mask =  SubstructureRedirectMask
@@ -198,7 +199,9 @@ wm_usage(char *progname)
 #endif
    printf("\t-use_dialog_mode  <free|const|const-horiz>\n");
    printf("\t-use_desktop_mode <decorated|plain>\n");
+   /*
    printf("\t-ping_handler     <string>\n");
+   */
 #ifdef STANDALONE
    printf("\t-titlebar_panel   <x11 geometry>\n");
 #endif
@@ -276,8 +279,14 @@ wm_usage(char *progname)
    printf("\tcomposite support                no\n");
 #endif
 
+#ifndef NO_PING
+   printf("\tping protocol support            yes\n");
+#else
+   printf("\tping protocol support            no\n");
+#endif
+
    printf("\nVisit http://matchbox.handhelds.org for more info.\n");
-   printf("(c) 2002 Matthew Allum\n");
+   printf("(c) 2004 Matthew Allum\n");
    exit(0);
 }
 
@@ -290,7 +299,7 @@ wm_load_config (Wm   *w,
    char *type;
    XrmValue value;
    
-   static int opTableEntries = 9;
+   static int opTableEntries = 8;
    static XrmOptionDescRec opTable[] = {
       {"-theme",       ".theme",           XrmoptionSepArg, (XPointer) NULL},
       {"-use_titlebar",".titlebar",        XrmoptionSepArg, (XPointer) NULL},
@@ -299,7 +308,9 @@ wm_load_config (Wm   *w,
       {"-use_lowlight",    ".lowlight",    XrmoptionSepArg, (XPointer) NULL},
       {"-use_dialog_mode", ".dialog",      XrmoptionSepArg, (XPointer) NULL},
       {"-use_desktop_mode",".desktop",     XrmoptionSepArg, (XPointer) NULL},
+      /*
       {"-ping_handler",    ".pinghandler",     XrmoptionSepArg, (XPointer) NULL},
+      */
       {"-titlebar_panel",  ".titlebarpanel", XrmoptionSepArg, (XPointer) NULL},
    };
    
@@ -432,6 +443,7 @@ wm_load_config (Wm   *w,
 	 }
      } 
 
+   /*
    if (XrmGetResource(rDB, "matchbox.pinghandler", "Matchbox.Pinghandler",
 		      &type, &value) == True)
    {
@@ -440,6 +452,7 @@ wm_load_config (Wm   *w,
      w->config->ping_handler[value.size] = '\0';
      dbg("%s() got ping handler :%s ", __func__, w->config->ping_handler);
    }
+   */
 
 #ifdef STANDALONE
    if (XrmGetResource(rDB, "matchbox.titlebarpanel", "Matchbox.Titlebarpanel",
@@ -515,6 +528,13 @@ wm_find_client(Wm *w, Window win, int mode)
 static Bool
 get_xevent_timed(Display* dpy, XEvent* event_return, struct timeval *tv)
 {
+
+  if (tv->tv_usec == 0 && tv->tv_sec == 0)
+    {
+      XNextEvent(dpy, event_return);
+      return True;
+    }
+
   XFlush(dpy);
 
   if (XPending(dpy) == 0) 
@@ -523,12 +543,14 @@ get_xevent_timed(Display* dpy, XEvent* event_return, struct timeval *tv)
       fd_set readset;
       FD_ZERO(&readset);
       FD_SET(fd, &readset);
+
       if (select(fd+1, &readset, NULL, NULL, tv) == 0) 
 	return False;
       else {
 	XNextEvent(dpy, event_return);
 	return True;
       }
+
     } else {
       XNextEvent(dpy, event_return);
       return True;
@@ -626,19 +648,32 @@ wm_event_loop(Wm* w)
     {
 
       tvt.tv_usec = 0;
-      tvt.tv_sec = 1;
-      /*
-      tvt.tv_usec = 500;
-      tvt.tv_sec = 0;
-      */
-      //            do {
+      tvt.tv_sec  = 0;
+
+#ifdef USE_LIBSN
+      if (w->sn_busy_cnt)
+	tvt.tv_sec = 1;
+#endif      
+
+#ifdef USE_MSG_WIN
+      if (w->msg_win_queue_head)
+	tvt.tv_sec = 1;
+#endif
+
+#ifdef USE_GCONF
+      if (w->gconf_client != NULL)
+	tvt.tv_sec = 1;
+#endif
+
+#ifndef NO_PING
+      if (w->n_active_ping_clients)
+	tvt.tv_sec = 1;
+#endif
 
       if (get_xevent_timed(w->dpy, &ev, &tvt))
 	{
 
-	  //XNextEvent(w->dpy, &ev);
-
-	switch (ev.type) 
+	  switch (ev.type) 
 	  {
 #ifdef USE_COMPOSITE
 	  case MapNotify:
@@ -666,9 +701,6 @@ wm_event_loop(Wm* w)
 	  case PropertyNotify:
 	    wm_handle_property_change(w, &ev.xproperty); break;
 #ifndef NO_KBD
-	  case KeymapNotify:
-	    /* keys_reinit(w); XXX why does this always get called ? */
-	    break;
 	  case MappingNotify:
 	    dbg("%s() got MappingNotify\n", __func__);
 	    XRefreshKeyboardMapping(&ev.xmapping);
@@ -695,13 +727,12 @@ wm_event_loop(Wm* w)
 	/* No X event poll checks here */
 #ifdef USE_LIBSN
 	if (w->sn_busy_cnt)
-	  wm_sn_timeout_check (w);
+	    wm_sn_timeout_check (w);
 #endif      
 
 #ifdef USE_MSG_WIN
 	if (w->msg_win_queue_head)
 	  {
-	    /* dbg("%s() checking message queue\n", __func__); */
 	    wm_msg_win_queue_process (w);
 	  }
 #endif
@@ -711,12 +742,14 @@ wm_event_loop(Wm* w)
 	  g_main_context_iteration (w->gconf_context, FALSE);
 #endif
 
-	/* check for hung apps every five seconds */
-	if (++hung_app_timer > 5)
+#ifndef NO_PING
+	/* check for hung apps every two seconds - they dont last long.. */
+	if (++hung_app_timer > 2 && w->n_active_ping_clients)
 	  {
 	    hung_app_timer = 0;
 	    ewmh_hung_app_check(w);
 	  }
+#endif
    
       }
 
@@ -1437,10 +1470,12 @@ wm_make_new_client(Wm *w, Window win)
 	       c = dialog_client_new(w, win, NULL);
 	       if (c == NULL) goto end;
 	     }
+
+#ifdef USE_MSG_WIN
 	   
 	   else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE])
 	     {
-#ifdef USE_MSG_WIN
+
 	       dbg("%s() got type message atom\n", __func__ );
 	       if (w->msg_win_queue_head == NULL)
 		 {
@@ -1465,13 +1500,22 @@ wm_make_new_client(Wm *w, Window win)
 		   if (value) XFree(value);
 		   return NULL;
 		 }
-#else
-	       fprintf(stderr, 
-		       "matchbox: This matchbox build lacks support for message windows.\n"
-		       "          To support them rebuild matchbox with --enable-message-wins.\n" 
-		       );
-#endif
+
 	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE_STATIC_0])
+	     {
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags ^= CLIENT_IS_MESSAGE_DIALOG|CLIENT_IS_MESSAGE_DIALOG_HI; 
+	     }
+	   else if (value[0] == w->atoms[WINDOW_TYPE_MESSAGE_STATIC_1])
+	     {
+		   c = dialog_client_new(w, win, NULL);
+		   if (c == NULL) goto end;
+		   c->flags ^= CLIENT_IS_MESSAGE_DIALOG|CLIENT_IS_MESSAGE_DIALOG_LO; 
+	     }
+
+#endif
 	   
 	 } 
      }
@@ -1986,7 +2030,11 @@ wm_toggle_desktop(Wm *w)
 	{
 	  if ( p->type == toolbar) 
 	    { XMapRaised(w->dpy, p->frame); }
-	  if ( p->type == dock && !(p->flags & CLIENT_DOCK_TITLEBAR)) 
+	  /* If panel is in titlebar. and wants to be hidden for desktop  */
+	  if ( p->type == dock && 
+	       ( !(p->flags & CLIENT_DOCK_TITLEBAR) 
+		 || (p->flags & CLIENT_DOCK_TITLEBAR_SHOW_ON_DESKTOP) )
+	       ) 
 	    XMapRaised(w->dpy, p->window);
 	  if (p->type == dialog && p->trans == NULL && p->mapped)
 	    {
@@ -2178,6 +2226,8 @@ wm_sn_timeout_check (Wm *w)
 {
   time_t now ;
 
+  dbg("%s() called\n", __func__);
+
   if (!w->sn_busy_cnt) return;
   
   now = time(NULL);
@@ -2553,6 +2603,8 @@ wm_msg_win_queue_add(Wm *w, Window win)
     
   if (data) XFree (data);
 
+  dbg("%s() timeout is %i\n", __func__, timeout);
+
   if (w->msg_win_queue_head == NULL)
     {
       dbg("%s() message queue is empty, adding new head\n", __func__);
@@ -2566,6 +2618,8 @@ wm_msg_win_queue_add(Wm *w, Window win)
   
   dbg("%s() queue has items, adding to back \n", __func__);
 
+
+
   while (tmp->next != NULL) tmp = tmp->next;
 
   tmp->next = malloc(sizeof(MsgWinQueue));
